fprintf('Reading data files...\n');
train_data=csvread('train_data_file.csv');
test_data=csvread('test_data_file.csv');
fprintf('Done reading data files\n');

assert(size(train_data,2)==size(test_data,2));%Both should have same number of metrics

M=size(train_data,2);%number of metrics
window_size=50;%a parameter that can be changed
train_CCVs=zeros(size(train_data,1),M*(M-1)/2);
test_CCVs=zeros(size(test_data,1),M*(M-1)/2);
CCV_distance=zeros(size(test_data,1),1);
CCV_distance(1:window_size-1)=NaN;

train_SDs=std(train_data);

fprintf('Building hypersphere...\n');
col=0;
for m1=1:M
	for m2=m1+1:M
		col=col+1;
		for end_row=window_size:size(train_data,1)
			start_row=end_row-window_size+1;
			X=train_data(start_row:end_row,m1);
			Y=train_data(start_row:end_row,m2);
			X=X-mean(X);
			Y=Y-mean(Y);
			train_CCVs(end_row,col)=sum(X.*Y)/train_SDs(m1)/train_SDs(m2)/(window_size-1);
			if isnan(train_CCVs(end_row,col))
				train_CCVs(end_row,col)=0;
			end
		end
	end
end
fprintf('Done building hypersphere\n');

fprintf('Computing CCVs of test data...\n');
col=0;
for m1=1:M
	for m2=m1+1:M
		col=col+1;
		for end_row=window_size:size(test_data,1)
			start_row=end_row-window_size+1;
			X=test_data(start_row:end_row,m1);
			Y=test_data(start_row:end_row,m2);
			X=X-mean(X);
			Y=Y-mean(Y);
			test_CCVs(end_row,col)=sum(X.*Y)/train_SDs(m1)/train_SDs(m2)/(window_size-1);
			if isnan(test_CCVs(end_row,col))
				test_CCVs(end_row,col)=0;
			end
		end
	end
end
fprintf('Done computing CCVs of test data\n');

fprintf('Computing nearest neighbor CCV distance...\n');
for data_point=window_size:size(test_CCVs,1)
	CCV_distance(data_point)=Inf;
	for neighbor=window_size:size(train_CCVs,1)
		%compute Euclidean distance
		dif=test_CCVs(data_point,:)-train_CCVs(neighbor,:);
		current_distance=sum(dif.*dif);
		if current_distance<CCV_distance(data_point)
			CCV_distance(data_point)=current_distance;
		end
	end
	CCV_distance(data_point)=sqrt(CCV_distance(data_point));
end
fprintf('Done computing nearest neighbor CCV distance...\n');